from typing import Mapping, Any


class CreateEvent:  # 创建迭代
    def __init__(self, sprit_id: str = "", title: str = "", start_time: int = 0, end_time: int = 0, **param):
        self.sprit_id = sprit_id
        self.title = title
        self.start_time = start_time
        self.end_time = end_time


class UpdateEvent:  # 更新迭代
    def __init__(self, sprit_id: str = "", old_title: str = "", new_title: str = "",
                 old_start_time: int = 0, new_start_time: int = 0, old_end_time: int = 0,
                 new_end_time: int = 0, **param):
        self.sprit_id = sprit_id
        self.old_title = old_title
        self.new_title = new_title
        self.old_start_time = old_start_time
        self.new_start_time = new_start_time
        self.old_end_time = old_end_time
        self.new_end_time = new_end_time


class RemoveEvent:  # 删除迭代
    def __init__(self, sprit_id: str = "", title: str = "", start_time: int = 0, end_time: int = 0, **param):
        self.sprit_id = sprit_id
        self.title = title
        self.start_time = start_time
        self.end_time = end_time


SpritEvents = CreateEvent | UpdateEvent | RemoveEvent | None


def parseSpritEvent(ev: Mapping[str, Mapping[str, Any]]) -> SpritEvents:
    if "CreateEvent" in ev:
        return CreateEvent(**ev["CreateEvent"])
    elif "UpdateEvent" in ev:
        return UpdateEvent(**ev["UpdateEvent"])
    else:
        return None



from typing import Mapping, Any

EVENT_SOURCE = int
EVENT_SOURCE_GITLAB: EVENT_SOURCE = 0
EVENT_SOURCE_GITHUB: EVENT_SOURCE = 1
EVENT_SOURCE_GITEA: EVENT_SOURCE = 2
EVENT_SOURCE_GITEE: EVENT_SOURCE = 3
EVENT_SOURCE_GOGS: EVENT_SOURCE = 4
EVENT_SOURCE_JIRA: EVENT_SOURCE = 5
EVENT_SOURCE_CONFLUENCE: EVENT_SOURCE = 6
EVENT_SOURCE_JENKINS: EVENT_SOURCE = 7


SOURCE_USER_POLICY = int
SOURCE_USER_POLICY_NONE: SOURCE_USER_POLICY = 0  # 未设置策略
SOURCE_USER_POLICY_DISCARD: SOURCE_USER_POLICY = 1  # 丢弃事件
SOURCE_USER_POLICY_MAPPING: SOURCE_USER_POLICY = 2  # 映射用户
SOURCE_USER_POLICY_SKIP_MAPPING: SOURCE_USER_POLICY = 3  # 跳过用户映射环节


class CreateEvent:  # 创建事件源
    def __init__(self, event_source_id: str = "", event_source: EVENT_SOURCE = EVENT_SOURCE_GITLAB, title: str = "", **params):
        self.event_source_id = event_source_id
        self.event_source = event_source
        self.title = title


class UpdateEvent:  # 更新事件源
    def __init__(self, event_source_id: str = "", event_source: EVENT_SOURCE = EVENT_SOURCE_GITLAB, old_title: str = "", new_title: str = "", **params):
        self.event_source_id = event_source_id
        self.event_source = event_source
        self.old_title = old_title
        self.new_title = new_title


class GetSecretEvent:  # 获取事件源秘钥
    def __init__(self, event_source_id: str = "", event_source: EVENT_SOURCE = EVENT_SOURCE_GITLAB, title: str = "", **params):
        self.event_source_id = event_source_id
        self.event_source = event_source
        self.title = title


class RemoveEvent:  # 删除事件源
    def __init__(self, event_source_id: str = "", event_source: EVENT_SOURCE = EVENT_SOURCE_GITLAB, title: str = "", **params):
        self.event_source_id = event_source_id
        self.event_source = event_source
        self.title = title


class SetSourceUserPolicyEvent:  # 设置事件源用户策略
    def __init__(self, event_source_id: str = "", event_source: EVENT_SOURCE = EVENT_SOURCE_GITLAB, title: str = "", source_user_name: str = "",
                 source_display_name: str = "", user_policy: SOURCE_USER_POLICY = SOURCE_USER_POLICY_NONE, map_user_id: str = "",
                 map_user_display_name: str = "", **params):
        self.event_source_id = event_source_id
        self.event_source = event_source
        self.title = title
        self.source_user_name = source_user_name
        self.source_display_name = source_display_name
        self.user_policy = user_policy
        self.map_user_id = map_user_id
        self.map_user_display_name = map_user_display_name


ExtEvents = CreateEvent | UpdateEvent | GetSecretEvent | RemoveEvent | SetSourceUserPolicyEvent | None


def parseExtEvent(ev: Mapping[str, Mapping[str, Any]]) -> ExtEvents:
    if "CreateEvent" in ev:
        return CreateEvent(**ev["CreateEvent"])
    elif "UpdateEvent" in ev:
        return UpdateEvent(**ev["UpdateEvent"])
    elif "GetSecretEvent" in ev:
        return GetSecretEvent(**ev["GetSecretEvent"])
    elif "RemoveEvent" in ev:
        return RemoveEvent(**ev["RemoveEvent"])
    elif "SetSourceUserPolicyEvent" in ev:
        return SetSourceUserPolicyEvent(**ev["SetSourceUserPolicyEvent"])
    else:
        return None



from typing import Mapping, Any

class CreateEvent:  # 创建机器人
    def __init__(self, robot_id: str = "", robot_name: str = "", **param):
        robot_id = robot_id
        robot_name = robot_name


class UpdateEvent:  # 更新机器人
    def __init__(self, robot_id: str = "", old_robot_name: str = "", new_robotname: str = "", **param):
        robot_id = robot_id
        old_robot_name = old_robot_name
        new_robotname = new_robotname


class RemoveEvent:  # 删除机器人
    def __init__(self, robot_id: str = "", robot_name: str = "", **param):
        robot_id = robot_id
        robot_name = robot_name


class AddAccessUserEvent:  # 新增访问用户
    def __init__(self, robot_id: str = "", robot_name: str = "", member_user_id: str = "", member_display_name: str = "", **param):
        robot_id = robot_id
        robot_name = robot_name
        member_user_id = member_user_id
        member_display_name = member_display_name


class RemoveAccessUserEvent:  # 删除访问用户
    def __init__(self, robot_id: str = "", robot_name: str = "", member_user_id: str = "", member_display_name: str = "", **param):
        robot_id = robot_id
        robot_name = robot_name
        member_user_id = member_user_id
        member_display_name = member_display_name


class RenewTokenEvent:  # 更新访问令牌
    def __init__(self, robot_id: str = "", robot_name: str = "", **param):
        robot_id = robot_id
        robot_name = robot_name

RobotEvents = CreateEvent|UpdateEvent|RemoveEvent|AddAccessUserEvent|RemoveAccessUserEvent|RenewTokenEvent|None

def parseRobotEvent(ev: Mapping[str, Mapping[str, Any]]) -> RobotEvents:
    if "CreateEvent" in ev:
        return CreateEvent(**ev["CreateEvent"])
    elif "UpdateEvent" in ev:
        return UpdateEvent(**ev["UpdateEvent"])
    elif "RemoveEvent" in ev:
        return RemoveEvent(**ev["RemoveEvent"])
    elif "AddAccessUserEvent" in ev:
        return AddAccessUserEvent(**ev["AddAccessUserEvent"])
    elif "RemoveAccessUserEvent" in ev:
        return RemoveAccessUserEvent(**ev["RemoveAccessUserEvent"])
    elif "RenewTokenEvent" in ev:
        return RenewTokenEvent(**ev["RenewTokenEvent"])
    else:
        return None


from typing import Mapping, Any


class AddRepoEvent:  # 创建仓库
    def __init__(self, repo_id: str = "", repo_url: str = "", **params):
        self.repo_id = repo_id
        self.repo_url = repo_url


class RemoveRepoEvent:  # 删除仓库
    def __init__(self, repo_id: str = "", repo_url: str = "", **params):
        self.repo_id = repo_id
        self.repo_url = repo_url


class CreateActionEvent:  # 创建执行命令
    def __init__(self, repo_id: str = "", repo_url: str = "", action_id: str = "", action_name: str = "", **params):
        self.repo_id = repo_id
        self.repo_url = repo_url
        self.action_id = action_id
        self.action_name = action_name


class UpdateActionEvent:  # 更新执行命令
    def __init__(self, repo_id: str, repo_url: str = "", action_id: str = "", action_name: str = "", **params):
        self.repo_id = repo_id
        self.repo_url = repo_url
        self.action_id = action_id
        self.action_name = action_name


class RemoveActionEvent:  # 删除执行命令
    def __init__(self, repo_id: str = "", repo_url: str = "", action_id: str = "", action_name: str = "", **params):
        self.repo_id = repo_id
        self.repo_url = repo_url
        self.action_id = action_id
        self.action_name = action_name


EarthlyEvents = AddRepoEvent | RemoveRepoEvent | CreateActionEvent | UpdateActionEvent | RemoveActionEvent | None


def parseEarthlyEvent(ev: Mapping[str, Mapping[str, Any]]) -> EarthlyEvents:
    if "AddRepoEvent" in ev:
        return AddRepoEvent(**ev["AddRepoEvent"])
    elif "RemoveRepoEvent" in ev:
        return RemoveRepoEvent(**ev["RemoveRepoEvent"])
    elif "CreateActionEvent" in ev:
        return CreateActionEvent(**ev["CreateActionEvent"])
    elif "UpdateActionEvent" in ev:
        return UpdateActionEvent(**ev["UpdateActionEvent"])
    elif "RemoveActionEvent" in ev:
        return RemoveActionEvent(**ev["RemoveActionEvent"])
    else:
        return None



from typing import Mapping, Any


class CreateSpaceEvent:  # 创建文档空间
    def __init__(self, doc_space_id: str = "", title: str = "", **params):
        self.doc_space_id = doc_space_id
        self.title = title


class UpdateSpaceEvent:  # 修改文档空间
    def __init__(self, doc_space_id: str = "", old_title: str = "", new_title: str = "", **params):
        self.doc_space_id = doc_space_id
        self.old_title = old_title
        self.new_title = new_title


class RemoveSpaceEvent:  # 删除文档空间
    def __init__(self, doc_space_id: str = "", title: str = "", **params):
        self.doc_space_id = doc_space_id
        self.title = title


class CreateDocEvent:  # 创建文档
    def __init__(self, doc_space_id: str = "", doc_space_name: str = "", doc_id: str = "", title: str = "", **params):
        self.doc_space_id = doc_space_id
        self.doc_space_name = doc_space_name
        self.doc_id = doc_id
        self.title = title


class UpdateDocEvent:  # 更新文档 (同一个用户同一个文档修改，不记录一个小时内的操作)
    def __init__(self, doc_space_id: str = "", doc_space_name: str = "", doc_id: str = "", old_title: str = "", new_title: str = "", **params):
        self.doc_space_id = doc_space_id
        self.doc_space_name = doc_space_name
        self.doc_id = doc_id
        self.old_title = old_title
        self.new_title = new_title


class MoveDocToRecycleEvent:  # 移动到回收站
    def __init__(self, doc_space_id: str = "", doc_space_name: str = "", doc_id: str = "", title: str = "", **params):
        self.doc_space_id = doc_space_id
        self.doc_space_name = doc_space_name
        self.doc_id = doc_id
        self.title = title


class RemoveDocEvent:  # 删除文档
    def __init__(self, doc_space_id: str = "", doc_space_name: str = "", doc_id: str = "", title: str = "", **params):
        self.doc_space_id = doc_space_id
        self.doc_space_name = doc_space_name
        self.doc_id = doc_id


class RecoverDocEvent:  # 恢复文档
    def __init__(self, doc_space_id: str = "", doc_space_name: str = "", doc_id: str = "", title: str = "", **params):
        self.doc_space_id = doc_space_id
        self.doc_space_name = doc_space_name
        self.doc_id = doc_id
        self.title = title


class WatchDocEvent:  # 关注文档
    def __init__(self, doc_space_id: str = "", doc_space_name: str = "", doc_id: str = "", title: str = "", **params):
        self.doc_space_id = doc_space_id
        self.doc_space_name = doc_space_name
        self.doc_id = doc_id
        self.title = title


class UnWatchDocEvent:  # 取消关注文档
    def __init__(self, doc_space_id: str = "", doc_space_name: str = "", doc_id: str = "", title: str = "", **params):
        self.doc_space_id = doc_space_id
        self.doc_space_name = doc_space_name
        self.doc_id = doc_id
        self.title = title


class MoveDocEvent:  # 移动文档
    def __init__(self, src_doc_space_id: str = "", src_doc_space_name: str = "", dest_doc_space_id: str = "",
                 dest_doc_space_name: str = "", doc_id: str = "", title: str = "", **params):
        self.src_doc_space_id = src_doc_space_id
        self.src_doc_space_name = src_doc_space_name
        self.dest_doc_space_id = dest_doc_space_id
        self.dest_doc_space_name = dest_doc_space_name
        self.doc_id = doc_id
        self.title = title


DocEvents = CreateSpaceEvent | UpdateSpaceEvent | RemoveSpaceEvent | CreateDocEvent | UpdateDocEvent | MoveDocToRecycleEvent | \
    RemoveDocEvent | RecoverDocEvent | WatchDocEvent | UnWatchDocEvent | MoveDocEvent | None


def parseDocEvent(ev: Mapping[str, Mapping[str, Any]]) -> DocEvents:
    if "CreateSpaceEvent" in ev:
        return CreateSpaceEvent(**ev["CreateSpaceEvent"])
    elif "UpdateSpaceEvent" in ev:
        return UpdateSpaceEvent(**ev["UpdateSpaceEvent"])
    elif "RemoveSpaceEvent" in ev:
        return RemoveSpaceEvent(**ev["RemoveSpaceEvent"])
    elif "CreateDocEvent" in ev:
        return CreateDocEvent(**ev["CreateDocEvent"])
    elif "UpdateDocEvent" in ev:
        return UpdateDocEvent(**ev["UpdateDocEvent"])
    elif "MoveDocToRecycleEvent" in ev:
        return MoveDocToRecycleEvent(**ev["MoveDocToRecycleEvent"])
    elif "RemoveDocEvent" in ev:
        return RemoveDocEvent(**ev["RemoveDocEvent"])
    elif "RecoverDocEvent" in ev:
        return RecoverDocEvent(**ev["RecoverDocEvent"])
    elif "WatchDocEvent" in ev:
        return WatchDocEvent(**ev["WatchDocEvent"])
    elif "UnWatchDocEvent" in ev:
        return UnWatchDocEvent(**ev["UnWatchDocEvent"])
    elif "MoveDocEvent " in ev:
        return MoveDocEvent(**ev["MoveDocEvent"])
    else:
        return None



from typing import Mapping, Any

from .project_book_shelf_events import parseBookShelfEvent, BookShelfEvents
from .project_doc_events import parseDocEvent, DocEvents
from .project_earthly_events import parseEarthlyEvent, EarthlyEvents
from .project_events import parseProjectEvent, ProjectEvents
from .project_ext_event_events import parseExtEvent, ExtEvents
from .project_gitee_events import parseGiteeEvent, GiteeEvents
from .project_gitlab_events import parseGitlabEvent, GitlabEvents
from .project_issue_event import parseIssueEvent, IssueEvents
from .project_robot_events import parseRobotEvent, RobotEvents
from .project_sprit_events import parseSpritEvent, SpritEvents


AllEvents = BookShelfEvents | DocEvents | EarthlyEvents | ProjectEvents | ExtEvents | GiteeEvents | GitlabEvents | IssueEvents | RobotEvents | SpritEvents | None


def parseAllEvent(ev: Mapping[str, Mapping[str, Mapping[str, Any]]]) -> AllEvents:
    if "ProjectEvent" in ev:
        return parseProjectEvent(ev["ProjectEvent"])
    elif "ProjectDocEvent" in ev:
        return parseDocEvent(ev["ProjectDocEvent"])
    elif "SpritEvent" in ev:
        return parseSpritEvent(ev["SpritEvent"])
    elif "IssueEvent" in ev:
        return parseIssueEvent(ev["IssueEvent"])
    elif "BookShelfEvent" in ev:
        return parseBookShelfEvent(ev["BookShelfEvent"])
    elif "ExtEvEvent" in ev:
        return parseExtEvent(ev["ExtEvEvent"])
    elif "GitlabEvent" in ev:
        return parseGitlabEvent(ev["GitlabEvent"])
    elif "GiteeEvent" in ev:
        return parseGiteeEvent(ev["GiteeEvent"])
    elif "RobotEvent" in ev:
        return parseRobotEvent(ev["RobotEvent"])
    elif "EarthlyEvent" in ev:
        return parseEarthlyEvent(ev["EarthlyEvent"])
    else:
        return None



from typing import Mapping, Any


class PushEvent:
    def __init__(self, **param):
        for k, v in param.items():
            self.__dict__[k] = v


class IssueEvent:
    def __init__(self, **param):
        for k, v in param.items():
            self.__dict__[k] = v


class PullRequestEvent:
    def __init__(self, **param):
        for k, v in param.items():
            self.__dict__[k] = v


class NoteEvent:
    def __init__(self, **param):
        for k, v in param.items():
            self.__dict__[k] = v


GiteeEvents = PushEvent | IssueEvent | PullRequestEvent | NoteEvent | None


def parseGiteeEvent(ev: Mapping[str, Mapping[str, Any]]) -> GiteeEvents:
    if "PushEvent" in ev:
        return PushEvent(**ev["PushEvent"])
    elif "IssueEvent" in ev:
        return IssueEvent(**ev["IssueEvent"])
    elif "PullRequestEvent" in ev:
        return PullRequestEvent(**ev["PullRequestEvent"])
    elif "NoteEvent" in ev:
        return NoteEvent(**ev["NoteEvent"])
    else:
        return None



from typing import Mapping, Any

ISSUE_TYPE = int
ISSUE_TYPE_TASK: ISSUE_TYPE = 0
ISSUE_TYPE_BUG: ISSUE_TYPE = 1

ISSUE_STATE = int
ISSUE_STATE_PLAN: ISSUE_STATE = 0
ISSUE_STATE_PROCESS: ISSUE_STATE = 1
ISSUE_STATE_CHECK: ISSUE_STATE = 2
ISSUE_STATE_CLOSE: ISSUE_STATE = 3


class CreateEvent:  # 创建issue
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, title: str = "", **param):
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.title = title


class UpdateEvent:  # 更新issue
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, old_title: str = "", new_title="", **param):
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.old_title = old_title
        self.new_title = new_title


class RemoveEvent:  # 删除issue
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, title: str = "", **param):
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.title = title


class AssignExecUserEvent:  # 指派执行者
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, title: str = "", exec_user_id: str = "",
                 exec_user_display_name: str = "", **param):
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.title = title
        self.exec_user_id = exec_user_id
        self.exec_user_display_name = exec_user_display_name


class AssignCheckUserEvent:  # 指派检查者
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, title: str = "",
                 check_user_id: str = "", check_user_display_name: str = "", **param):
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.title = title
        self.check_user_id = check_user_id
        self.check_user_display_name = check_user_display_name


class ChangeStateEvent:  # 修改状态
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, title: str = "",
                 old_state: ISSUE_STATE = ISSUE_STATE_PLAN, new_state: ISSUE_STATE = ISSUE_STATE_PLAN,  **param):
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.title = title
        self.old_state = old_state
        self.new_state = new_state


class LinkSpritEvent:  # 关联到迭代
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, title: str = "",
                 sprit_id: str = "", sprit_title: str = "",  **param):
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.title = title
        self.sprit_id = sprit_id
        self.sprit_title = sprit_title


class CancelLinkSpritEvent:  # 取消关联到迭代
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, title: str = "",
                 sprit_id: str = "", sprit_title: str = "", **param) -> None:
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.title = title
        self.sprit_id = sprit_id
        self.sprit_title = sprit_title


class SetStartTimeEvent:  # 设置开始时间
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, title: str = "", start_time: int = 0, **param):
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.title = title
        self.start_time = start_time


class CancelStartTimeEvent:  # 取消开始时间
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, title: str = "", **param):
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.title = title


class SetEndTimeEvent:  # 设置结束时间
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, title: str = "", end_time: int = 0, **param):
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.title = title
        self.end_time = end_time


class CancelEndTimeEvent:  # 取消结束时间
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, title: str = "", **param):
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.title = title


class SetEstimateMinutesEvent:  # 设置预估时间
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, title: str = "", estimate_minutes: int = 0, **param):
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.title = title
        self.estimate_minutes = estimate_minutes


class CancelEstimateMinutesEvent:  # 取消预估时间
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, title: str = "", **param):
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.title = title


class SetRemainMinutesEvent:  # 设置剩余时间
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, title: str = "",
                 remain_minutes: int = 0, has_spend_minutes: bool = False, spend_minutes: int = 0, **param):
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.title = title
        self.remain_minutes = remain_minutes
        self.has_spend_minutes = has_spend_minutes
        self.spend_minutes = spend_minutes


class CancelRemainMinutesEvent:  # 取消剩余时间
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, title: str = "", **param):
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.title = title


class CreateSubIssueEvent:  # 增加子工单
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, issue_title: str = "", title: str = "", **param):
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.issue_title = issue_title
        self.title = title


class UpdateSubIssueEvent:  # 更新子工单
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, issue_title: str = "",
                 old_title: str = "", new_title: str = "", **param):
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.issue_title = issue_title
        self.old_title = old_title
        self.new_title = new_title


class UpdateSubIssueStateEvent:  # 更新子工单状态
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, issue_title: str = "", title: str = "", done: bool = False, **param):
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.issue_title = issue_title
        self.title = title
        self.done = done


class RemoveSubIssueEvent:  # 删除子工单
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, issue_title: str = "", title: str = "", **param):
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.issue_title = issue_title
        self.title = title


class AddDependenceEvent:  # 增加依赖
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, issue_title: str = "",
                 depend_issue_id: str = "", depend_issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, depend_issue_title: str = "",
                 **param):
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.issue_title = issue_title
        self.depend_issue_id = depend_issue_id
        self.depend_issue_type = depend_issue_type
        self.depend_issue_title = depend_issue_title


class RemoveDependenceEvent:  # 删除依赖
    def __init__(self, issue_id: str = "", issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, issue_title: str = "",
                 depend_issue_id: str = "", depend_issue_type: ISSUE_TYPE = ISSUE_TYPE_TASK, depend_issue_title: str = "",
                 **param):
        self.issue_id = issue_id
        self.issue_type = issue_type
        self.issue_title = issue_title
        self.depend_issue_id = depend_issue_id
        self.depend_issue_type = depend_issue_type
        self.depend_issue_title = depend_issue_title


IssueEvents = CreateEvent | UpdateEvent | RemoveEvent | AssignExecUserEvent | AssignCheckUserEvent | ChangeStateEvent | LinkSpritEvent | CancelLinkSpritEvent | SetStartTimeEvent | CancelStartTimeEvent | SetEndTimeEvent | CancelEndTimeEvent | SetEstimateMinutesEvent | CancelEstimateMinutesEvent | SetRemainMinutesEvent | CancelRemainMinutesEvent | CreateSubIssueEvent | UpdateSubIssueEvent | UpdateSubIssueStateEvent | RemoveSubIssueEvent | AddDependenceEvent | RemoveDependenceEvent | None


def parseIssueEvent(ev: Mapping[str, Mapping[str, Any]]) -> IssueEvents:
    if "CreateEvent" in ev:
        return CreateEvent(**ev["CreateEvent"])
    elif "UpdateEvent" in ev:
        return UpdateEvent(**ev["UpdateEvent"])
    elif "RemoveEvent" in ev:
        return RemoveEvent(**ev["RemoveEvent"])
    elif "AssignExecUserEvent" in ev:
        return AssignExecUserEvent(**ev["AssignExecUserEvent"])
    elif "AssignCheckUserEvent" in ev:
        return AssignCheckUserEvent(**ev["AssignCheckUserEvent"])
    elif "ChangeStateEvent" in ev:
        return ChangeStateEvent(**ev["ChangeStateEvent"])
    elif "LinkSpritEvent" in ev:
        return LinkSpritEvent(**ev["LinkSpritEvent"])
    elif "SetStartTimeEvent" in ev:
        return SetStartTimeEvent(**ev["SetStartTimeEvent"])
    elif "CancelStartTimeEvent" in ev:
        return CancelStartTimeEvent(**ev["CancelStartTimeEvent"])
    elif "SetEndTimeEvent" in ev:
        return SetEndTimeEvent(**ev["SetEndTimeEvent"])
    elif "CancelEndTimeEvent" in ev:
        return CancelEndTimeEvent(**ev["CancelEndTimeEvent"])
    elif "SetEstimateMinutesEvent" in ev:
        return SetEstimateMinutesEvent(**ev["SetEstimateMinutesEvent"])
    elif "CancelEstimateMinutesEvent" in ev:
        return CancelEstimateMinutesEvent(**ev["CancelEstimateMinutesEvent"])
    elif "SetRemainMinutesEvent" in ev:
        return SetRemainMinutesEvent(**ev["SetRemainMinutesEvent"])
    elif "CancelRemainMinutesEvent" in ev:
        return CancelRemainMinutesEvent(**ev["CancelRemainMinutesEvent"])
    elif "CreateSubIssueEvent" in ev:
        return CreateSubIssueEvent(**ev["CreateSubIssueEvent"])
    elif "UpdateSubIssueEvent" in ev:
        return UpdateSubIssueEvent(**ev["UpdateSubIssueEvent"])
    elif "UpdateSubIssueStateEvent" in ev:
        return UpdateSubIssueStateEvent(**ev["UpdateSubIssueStateEvent"])
    elif "RemoveSubIssueEvent" in ev:
        return RemoveSubIssueEvent(**ev["RemoveSubIssueEvent"])
    elif "AddDependenceEvent" in ev:
        return AddDependenceEvent(**ev["AddDependenceEvent"])
    elif "RemoveDependenceEvent" in ev:
        return RemoveDependenceEvent(**ev["RemoveDependenceEvent"])
    else:
        return None



from typing import Mapping, Any

APP_OPEN_TYPE = int
OPEN_TYPE_BROWSER: APP_OPEN_TYPE = 0
OPEN_TYPE_INNER: APP_OPEN_TYPE = 1


class CreateProjectEvent:  # 创建项目
    pass


class UpdateProjectEvent:  # 设置项目
    def __init__(self, new_project_name: str = "", **params):
        # 项目名称
        self.new_project_name = new_project_name


class OpenProjectEvent:  # 打开项目
    pass


class CloseProjectEvent:  # 关闭项目
    pass


class RemoveProjectEvent:  # 删除项目
    pass


class GenInviteEvent:  # 创建邀请码
    pass


class JoinProjectEvent:  # 加入项目
    pass


class LeaveProjectEvent:  # 离开项目
    pass


class CreateRoleEvent:  # 创建角色
    def __init__(self, role_id: str = "", role_name: str = "", **params):
        self.role_id = role_id
        self.role_name = role_name


class UpdateRoleEvent:  # 更新角色
    def __init__(self, role_id: str = "", old_role_name: str = "", new_role_name: str = "", **params):
        self.role_id = role_id
        self.old_role_name = old_role_name
        self.new_role_name = new_role_name


class RemoveRoleEvent:  # 删除角色
    def __init__(self, role_id: str = "", role_name: str = "", **params):
        self.role_id = role_id
        self.role_name = role_name


class UpdateProjectMemberEvent:  # 更新成员
    def __init__(self, member_user_id: str = "", old_member_display_name: str = "", new_member_display_name: str = "", **params):
        self.member_user_id = member_user_id
        self.old_member_display_name = old_member_display_name
        self.new_member_display_name = new_member_display_name


class RemoveProjectMemberEvent:  # 删除成员
    def __init__(self, member_user_id: str = "", member_display_name: str = "", **params):
        self.member_user_id = member_user_id
        self.member_display_name = member_display_name


class SetProjectMemberRoleEvent:  # 设置成员角色
    def __init__(self, role_id: str = "", role_name: str = "", member_user_id: str = "", member_display_name: str = "", **params):
        self.role_id = role_id
        self.role_name = role_name
        self.member_user_id = member_user_id
        self.member_display_name = member_display_name


class UploadWorkSnapShotEvent:  # 工作快照
    def __init__(self, fs_id: str = "", file_id: str = "", thumb_file_id: str = "", **params):
        self.fs_id = fs_id
        self.file_id = file_id
        self.thumb_file_id = thumb_file_id


class CreateChannelEvent:  # 创建频道
    def __init__(self, channel_id: str = "", channel_name: str = "", pub_channel: bool = False, **params):
        self.channel_id = channel_id
        self.channel_name = channel_name
        self.pub_channel = pub_channel


class UpdateChannelEvent:  # 更新频道
    def __init__(self, channel_id: str = "", old_channel_name: str = "", old_pub_channel: bool = False, new_channel_name: str = "", new_pub_channel: bool = False, **params):
        self.channel_id = channel_id
        self.old_channel_name = old_channel_name
        self.old_pub_channel = old_pub_channel
        self.new_channel_name = new_channel_name
        self.new_pub_channel = new_pub_channel


class OpenChannelEvent:  # 打开频道
    def __init__(self, channel_id: str = "", channel_name: str = "", **params):
        self.channel_id = channel_id
        self.channel_name = channel_name


class CloseChannelEvent:  # 关闭频道
    def __init__(self, channel_id: str = "", channel_name: str = "", **params):
        self.channel_id = channel_id
        self.channel_name = channel_name


class RemoveChannelEvent:  # 删除频道
    def __init__(self, channel_id: str = "", channel_name: str = "", **params):
        self.channel_id = channel_id
        self.channel_name = channel_name


class AddChannelMemberEvent:  # 增加频道成员
    def __init__(self, channel_id: str = "", channel_name: str = "", member_user_id: str = "", member_display_name: str = "", **params):
        self.channel_id = channel_id
        self.channel_name = channel_name
        self.member_user_id = member_user_id
        self.member_display_name = member_display_name


class RemoveChannelMemberEvent:  # 删除频道成员
    def __init__(self, channel_id: str = "", channel_name: str = "", member_user_id: str = "", member_display_name: str = "", **params):
        self.channel_id = channel_id
        self.channel_name = channel_name
        self.member_user_id = member_user_id
        self.member_display_name = member_display_name


class CreateAppraiseEvent:  # 创建评估
    def __init__(self, appraise_id: str = "", title: str = "", **params):
        self.appraise_id = appraise_id
        self.title = title


class AddProjectAppEvent:  # 增加项目应用
    def __init__(self, app_id: str = "", app_name: str = "", app_url: str = "", app_open_type: APP_OPEN_TYPE = OPEN_TYPE_BROWSER, **params):
        self.app_id = app_id
        self.app_name = app_name
        self.app_url = app_url
        self.app_open_type = app_open_type


class RemoveProjectAppEvent:  # 删除项目应用
    def __init__(self, app_id: str = "", app_name: str = "", **params):
        self.app_id = app_id
        self.app_name = app_name


ProjectEvents = CreateProjectEvent | UpdateProjectEvent | OpenProjectEvent | CloseProjectEvent | RemoveProjectEvent | GenInviteEvent | JoinProjectEvent | LeaveProjectEvent | CreateRoleEvent | UpdateRoleEvent | RemoveRoleEvent | UpdateProjectMemberEvent | RemoveProjectMemberEvent | SetProjectMemberRoleEvent | UploadWorkSnapShotEvent | CreateChannelEvent | UpdateChannelEvent | OpenChannelEvent | CloseChannelEvent | RemoveChannelEvent | AddChannelMemberEvent | RemoveChannelMemberEvent | CreateAppraiseEvent | AddProjectAppEvent | RemoveProjectAppEvent | None


def parseProjectEvent(ev: Mapping[str, Mapping[str, Any]]) -> ProjectEvents:
    if "CreateProjectEvent" in ev:
        return CreateProjectEvent()
    elif "UpdateProjectEvent" in ev:
        return UpdateProjectEvent(**ev["UpdateProjectEvent"])
    elif "OpenProjectEvent" in ev:
        return OpenProjectEvent()
    elif "CloseProjectEvent" in ev:
        return CloseProjectEvent()
    elif "RemoveProjectEvent" in ev:
        return RemoveProjectEvent()
    elif "GenInviteEvent" in ev:
        return GenInviteEvent()
    elif "JoinProjectEvent" in ev:
        return JoinProjectEvent()
    elif "LeaveProjectEvent" in ev:
        return LeaveProjectEvent()
    elif "CreateRoleEvent" in ev:
        return CreateRoleEvent(**ev["CreateRoleEvent"])
    elif "UpdateRoleEvent" in ev:
        return UpdateRoleEvent(**ev["UpdateRoleEvent"])
    elif "RemoveRoleEvent" in ev:
        return RemoveRoleEvent(**ev["RemoveRoleEvent"])
    elif "UpdateProjectMemberEvent" in ev:
        return UpdateProjectMemberEvent(**ev["UpdateProjectMemberEvent"])
    elif "RemoveProjectMemberEvent" in ev:
        return RemoveProjectMemberEvent(**ev["RemoveProjectMemberEvent"])
    elif "SetProjectMemberRoleEvent" in ev:
        return SetProjectMemberRoleEvent(**ev["SetProjectMemberRoleEvent"])
    elif "UploadWorkSnapShotEvent" in ev:
        return UploadWorkSnapShotEvent(**ev["UploadWorkSnapShotEvent"])
    elif "CreateChannelEvent" in ev:
        return CreateChannelEvent(**ev["CreateChannelEvent"])
    elif "UpdateChannelEvent" in ev:
        return UpdateChannelEvent(**ev["UpdateChannelEvent"])
    elif "OpenChannelEvent" in ev:
        return OpenChannelEvent(**ev["OpenChannelEvent"])
    elif "CloseChannelEvent" in ev:
        return CloseChannelEvent(**ev["CloseChannelEvent"])
    elif "RemoveChannelEvent" in ev:
        return RemoveChannelEvent(**ev["RemoveChannelEvent"])
    elif "AddChannelMemberEvent" in ev:
        return AddChannelMemberEvent(**ev["AddChannelMemberEvent"])
    elif "RemoveChannelMemberEvent" in ev:
        return RemoveChannelMemberEvent(**ev["RemoveChannelMemberEvent"])
    elif "CreateAppraiseEvent" in ev:
        return CreateAppraiseEvent(**ev["CreateAppraiseEvent"])
    elif "AddProjectAppEvent" in ev:
        return AddProjectAppEvent(**ev["AddProjectAppEvent"])
    elif "RemoveProjectAppEvent" in ev:
        return RemoveProjectAppEvent(**ev["RemoveProjectAppEvent"])
    else:
        return None



from typing import Mapping, Any

class BuildEvent:
    def __init__(self, **param):
        for k, v in param.items():
            self.__dict__[k] = v


class CommentEvent:
    def __init__(self, **param):
        for k, v in param.items():
            self.__dict__[k] = v


class IssueEvent:
    def __init__(self, **param):
        for k, v in param.items():
            self.__dict__[k] = v


class JobEvent:
    def __init__(self, **param):
        for k, v in param.items():
            self.__dict__[k] = v


class MergeRequestEvent:
    def __init__(self, **param):
        for k, v in param.items():
            self.__dict__[k] = v


class PipelineEvent:
    def __init__(self, **param):
        for k, v in param.items():
            self.__dict__[k] = v


class PushEvent:
    def __init__(self, **param):
        for k, v in param.items():
            self.__dict__[k] = v


class TagEvent:
    def __init__(self, **param):
        for k, v in param.items():
            self.__dict__[k] = v


class WikiEvent:
    def __init__(self, **param):
        for k, v in param.items():
            self.__dict__[k] = v

GitlabEvents = BuildEvent|CommentEvent|IssueEvent|JobEvent|MergeRequestEvent|PipelineEvent|PushEvent|TagEvent|WikiEvent|None

def parseGitlabEvent(ev: Mapping[str, Mapping[str, Any]]) -> GitlabEvents:
    if "BuildEvent" in ev:
        return BuildEvent(**ev["BuildEvent"])
    elif "CommentEvent" in ev:
        return CommentEvent(**ev["CommentEvent"])
    elif "IssueEvent" in ev:
        return IssueEvent(**ev["IssueEvent"])
    elif "JobEvent" in ev:
        return JobEvent(**ev["JobEvent"])
    elif "MergeRequestEvent" in ev:
        return MergeRequestEvent(**ev["MergeRequestEvent"])
    elif "PipelineEvent" in ev:
        return PipelineEvent(**ev["PipelineEvent"])
    elif "PushEvent" in ev:
        return PushEvent(**ev["PushEvent"])
    elif "TagEvent" in ev:
        return TagEvent(**ev["TagEvent"])
    elif "WikiEvent" in ev:
        return WikiEvent(**ev["WikiEvent"])
    else:
        return None




from typing import Mapping, Any


class AddBookEvent:  # 增加书本
    def __init__(self, book_id: str = "", book_title: str = "", **params):
        self.book_id = book_id
        self.book_title = book_title


class RemoveBookEvent:  # 删除书本
    def __init__(self, book_id: str = "", book_title: str = "", **params):
        self.book_id = book_id
        self.book_title = book_title


BookShelfEvents = AddBookEvent | RemoveBookEvent | None


def parseBookShelfEvent(ev: Mapping[str, Mapping[str, Any]]) -> BookShelfEvents:
    if "AddBookEvent" in ev:
        return AddBookEvent(**ev["AddBookEvent"])
    elif "RemoveBookEvent" in ev:
        return RemoveBookEvent(**ev["RemoveBookEvent"])
    else:
        return None
